apoisson = function(x,y,XB, nseg = 10, bdeg = 3, pord = 2, lambda = 1, 
              plot = T, se = 2, show = T)
{
  # Function ps.binomial: smooths scattered binary response data with P-splines.
  # Input: x = abcissae of data.
  # Input: y = binary response.
  # Input: X = B spline basis obtained using the x.
  # Input: nseg = number of intervals for B-splines.
  # Input: bdeg = degree of B-splines.
  # Input: pord = order of difference penalty.
  # Input: lambda = smoothness parameter.
  #
  # Output: object list, including AIC = deviance + 2*trace(Hat).
  #
  # Side effect: a plot of (x,y) and the estimated probability with twice se bands.
  
	
	m = length(x)
	xl = min(x)
	xr = max(x)
	
 	B=XB    # B-spline basis
	obs=y

  # Construct penalty stuff
	D=diff(diag(ncol(B)), diff = pord)
	P=t(D)%*%D
	it = 0	
	beta = solve(t(B)%*%B+lambda*P)%*%t(B)%*%log(1+y)
#	beta=rep(1,ncol(B))
	err=10
	eps=1e-7

    eta = as.vector(B%*%beta)
	fitted = w = mu = as.vector(exp(eta))
	dev = -2 * sum(y*log(mu)-mu) 

	
  # Compute beta by iteration
	while (err>eps) {
		it=it+1
		z=eta+((y-mu)/w)
		WB=w*B
		
		covar = solve(t(B)%*%WB+lambda*P)
		beta = as.vector(covar%*%t(WB)%*%z)
	   eta = as.vector(B%*%beta)
     	fitted = w = mu = as.vector(exp(eta))
       dev.old = dev
		dev = -2 * sum(y*log(mu)-mu) 
		err = max(abs(dev.old-dev))
      # cat("Iteration ",it,"\n")
	}

  # Compute AIC
	e =eps
	tr.h=sum(diag(covar%*%t(B)%*%WB))
	eff.dim=tr.h
	aic = dev + 2 * tr.h	
  	nm = nrow(B)
   	AIC = dev+2*eff.dim ; 
	BIC = dev+log(nm)*eff.dim
	
	silent=0    
	if (!silent){
	#    cat("\nNumber of iterations:",it," ;  Effective dimension:",eff.dim,"\n")
    	#    cat("Deviance:",dev," ;  AIC:",AIC," ;  BIC:",BIC,"\n")
	}
  # Compute curve on grid
	xgrid = seq(xl, xr, length = 100)
	Bu = bsplbase(xgrid, c(xl,xr,nseg,bdeg))
	zu = Bu %*% beta
	ygrid = exp(zu)	# Plot data and fit
	
  # Return list
	pp= list(x = x, y = y, muhat = mu, nseg = nseg, 
            bdeg = bdeg, pord = pord, lambda = lambda, 
            xgrid = xgrid, ygrid = ygrid,covar=covar, effdim = tr.h, dispersion = 1, 
            family = "poisson", link = "log", coef = beta, fitted=fitted,
            deviance=dev,aic = AIC,bic=BIC)
	class(pp) = "pspfit"
	return(pp)
}

