### Alternative definition of B-splines basis ###
### ----------------------------------------- ### 
# Fast computation of a B-spline basis of degree "deg", at positions "x",
# on a uniform grid with "ndx" intervals between "x0" and "x1" along Eilers WITHOUT tied knots.
bsplbase <- function(x, bpars){
  x0 = bpars[1]; x1 = bpars[2]; ndx = bpars[3]; deg = bpars[4]
  x = matrix(x,ncol=1)
  if ((min(x) < x0) | (max(x) > x1)){
    print("Some elements of x out of bounds !!")
    return(0)
  }
  dx = (x1 - x0) / ndx
  tt = matrix(x0 + dx * ((-deg):(ndx-1)),nrow=1)
  TT = matrix(1,nrow=nrow(x),ncol=1)%*%tt
  X = x%*%matrix(1,nrow=nrow(tt),ncol=ncol(tt))
  D = (X - TT) / dx;
  B = t(diff(t(cbind(0*x, D <= 1))))
  r = c(2:length(tt),1)
  for (k in 1:deg){
    B = (D * B + (k + 1 - D) * B[, r]) / k
  }
  return(B)
}
